# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import enum

import winrt.system
from winui3._winui3_microsoft_windows_pushnotifications import (
    PushNotificationCreateChannelStatus,
    PushNotificationChannel,
    PushNotificationCreateChannelResult,
    PushNotificationManager,
    PushNotificationReceivedEventArgs,
)

__all__ = [
    "PushNotificationChannelStatus",
    "PushNotificationCreateChannelStatus",
    "PushNotificationChannel",
    "PushNotificationCreateChannelResult",
    "PushNotificationManager",
    "PushNotificationReceivedEventArgs",
]

class PushNotificationChannelStatus(enum.IntEnum):
    IN_PROGRESS = 0
    IN_PROGRESS_RETRY = 1
    COMPLETED_SUCCESS = 2
    COMPLETED_FAILURE = 3

